/* =============================================================================
 *
 *  Description: This is a C++ implementation for Thread input_read
 *
 * -----------------------------------------------------------------------------
 *  Comments:
 *
 * ===========================================================================*/

#include "input_read.h"
#include <new>
#include "msg.h"
#include "globals.h"

#pragma file_attr("OS_Component=Threads")
#pragma file_attr("Threads")

/******************************************************************************
 *  input_read Run Function (input_read's main{})
 */
 
typedef short SampleWord;

 
void
input_read::Run()
{
    // TODO - Put the thread's "main" Initialization HERE
	VDK::DeviceDescriptor dd = VDK::OpenDevice(kADC0, "R16"); // eszkz (AD) megnyitsa olvassra 16 bites adatokra
	
	void *pBuffer1 = new SampleWord[2*SAMPLEPAIRS_PER_BUFFER];    
    void *pBuffer2 = new SampleWord[2*SAMPLEPAIRS_PER_BUFFER];    
    unsigned char current_buffer = 1;

 	// zenetobjektumok ltrehozsa
 	// msg1 zenet tartalma az 1-es buffer (pBuffer1) ltal mutatott terlet
 	// msg2 zenet tartalma a  2-es buffer (pBuffer2) ltal mutatott terlet
    VDK::MessageID msg1 = VDK::CreateMessage(1, SAMPLEPAIRS_PER_BUFFER, pBuffer1); 
 	VDK::MessageID msg2 = VDK::CreateMessage(1, SAMPLEPAIRS_PER_BUFFER, pBuffer2);
 	
 	// els adatblokk beolvassa   
    VDK::SyncRead(dd, (char*) pBuffer1, SAMPLEPAIRS_PER_BUFFER*2*sizeof(SampleWord), 0);
    
    // Megjegyzs: az olvass mvelet - VDK::SyncRead(dd...) - csak addig blokkolja a kdot,
    // amg a DMA mveletet elkszti a processzor (vagy, amg az elz DMA mvelet folyamatban van). 
    // A DMA ezt kveten automatikusan olvassa az adatokat a httrben.     
    
    while (1)
    {
        // TODO - Put the thread's "main" body HERE
		if(1 == current_buffer) // az 1-es az ppen aktulis adatokat tartalmaz buffer
		{
        	// AD olvass kezdemnyezse a 2-es bufferbe
        	VDK::SyncRead(dd, (char*) pBuffer2, SAMPLEPAIRS_PER_BUFFER*2*sizeof(SampleWord), 0);
        	current_buffer = 2; // 2-es lesz a kvetkez aktulis buffer (mg feltlts alatt)
        	VDK::PostMessage(kmt, msg1, INPUT_DATA_CHANNEL); // a mr feltlttt 1-es buffert elkldi zenetben
        	msg1 = VDK::PendMessage(INPUT_DATA_BACKCHANNEL, 0); // vr, amg a fprogram vissza nem jelez
		}
		else // a 2-es az ppen aktulis adatokat tartalmaz buffer
		{
        	// AD olvass kezdemnyezse az 1-es bufferbe
        	VDK::SyncRead(dd, (char*) pBuffer1, SAMPLEPAIRS_PER_BUFFER*2*sizeof(SampleWord), 0);
        	current_buffer = 1; // 1-es lesz a kvetkez aktulis buffer (mg feltlts alatt)
        	VDK::PostMessage(kmt, msg2, INPUT_DATA_CHANNEL); // a mr feltlttt 2-es buffert elkldi zenetben
        	msg2 = VDK::PendMessage(INPUT_DATA_BACKCHANNEL, 0); // vr, amg a fprogram vissza nem jelez
		}
		
        // Use a "break" instruction to exit the "while (1)" loop
    }

    // TODO - Put the thread's exit from "main" HERE
	// A thread is automatically Destroyed when it exits its run function
}

/******************************************************************************
 *  input_read Error Handler
 */
 
int
input_read::ErrorHandler()
{
    /* TODO - Put this thread's error handling code HERE */

    /* The default ErrorHandler (called below)  raises
     * a kernel panic and stops the system */
    return (VDK::Thread::ErrorHandler());
}

/******************************************************************************
 *  input_read Constructor
 */
 
input_read::input_read(VDK::Thread::ThreadCreationBlock &tcb)
    : VDK::Thread(tcb)
{
    /* This routine does NOT run in new thread's context.  Any non-static thread
     *   initialization should be performed at the beginning of "Run()."
     */

    // TODO - Put code to be executed when this thread has just been created HERE
    m_main_thread = kmt;

}

/******************************************************************************
 *  input_read Destructor
 */
 
input_read::~input_read()
{
    /* This routine does NOT run in the thread's context.  Any VDK API calls
     *   should be performed at the end of "Run()."
     */

    // TODO - Put code to be executed just before this thread is destroyed HERE

}

/******************************************************************************
 *  input_read Externally Accessible, Pre-Constructor Create Function
 */
 
VDK::Thread*
input_read::Create(VDK::Thread::ThreadCreationBlock &tcb)
{
    /* This routine does NOT run in new thread's context.  Any non-static thread
     *   initialization should be performed at the beginning of "Run()."
     */

    	return new (tcb) input_read(tcb);
}

/* ========================================================================== */
